import json
import logging
import os
import shutil
import subprocess
import sys
from urllib import request
from urllib.parse import quote, urlparse

logging.basicConfig(level=logging.WARNING)


def download_file(url, save_path):
    """
    下载文件
    :param url: URL
    :param save_path: 保存文件名
    :return:
    """
    try:
        content = request.urlopen(url)
        basename = os.path.dirname(save_path)
        if not os.path.exists(basename):
            os.makedirs(basename)
        with open(save_path, 'wb') as f:
            f.write(content.read())
        return 'ok'
    except Exception as e:
        return e


def execute(command, cwd='/'):
    """
    执行本地命令
    :param command: 命令
    :param cwd: 执行环境
    :return: 输出，错误信息(bytes类型)
    """
    sub = subprocess.Popen(command, cwd=cwd, stdin=subprocess.PIPE, stdout=sys.stdout,
                           stderr=subprocess.STDOUT, shell=True)
    sub.communicate()


def delete_dir_then_move_dir(file_from, file_to):
    """
    删除文件夹并移动新文件夹替换之
    :param file_from:
    :param file_to:
    :return:
    """
    if os.path.exists(file_from):
        if os.path.exists(file_to):
            shutil.rmtree(file_to)
        shutil.move(file_from, file_to)


def delete_dir_then_copy_dir(file_from, file_to, symlinks=False, ignore=None):
    """
    删除文件夹并复制新文件夹替换之
    :param file_from:
    :param file_to:
    :param symlinks:
    :param ignore:
    :return:
    """
    if os.path.exists(file_from):
        if os.path.exists(file_to):
            shutil.rmtree(file_to)
        try:
            shutil.copytree(file_from, file_to, symlinks, ignore)
        except Exception as e:
            if isinstance(e, shutil.Error):
                pass


def make_dir(target_dir):
    """
    递归新建目录
    :param target_dir:
    :return:
    """
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)


def remake_dir(target_dir):
    """
    删除并重建目录
    :param target_dir:
    :return:
    """
    if os.path.exists(target_dir):
        shutil.rmtree(target_dir)
    os.makedirs(target_dir)


def pull_git_project(project_dir, git_branch):
    project_branch_dir = os.path.join(project_dir, git_branch)
    execute('git checkout .', project_branch_dir)
    execute('git clean -xfd ', project_branch_dir)
    execute('git pull origin %s' % (git_branch), project_branch_dir)


def clone_git_project(git_url, git_branch, clone_dir, origin_object_branch="master"):
    git_origin_object_branch_dir = os.path.join(clone_dir, origin_object_branch)
    clone_branch_dir = os.path.join(clone_dir, git_branch)
    if os.path.exists(clone_branch_dir) == False:
        make_dir(clone_branch_dir)

    if os.path.exists(git_origin_object_branch_dir):
        execute('git init', clone_branch_dir)
        execute('git remote add origin %s' % (git_url), clone_branch_dir)
        execute("echo ../../../%s/.git/objects/ > .git/objects/info/alternates" % (origin_object_branch),
                clone_branch_dir)
        execute("git pull origin %s:%s" % (git_branch, git_branch), clone_branch_dir)
        execute("git checkout %s" % (git_branch), clone_branch_dir)
    else:
        execute('git clone -b %s %s %s' % (origin_object_branch, git_url, origin_object_branch), clone_dir)
        clone_git_project(git_url, git_branch, clone_dir, origin_object_branch)


def sync_project_git_branch_dir(git_url, project_dir, branch_name):
    if os.path.exists(project_dir) == False:
        make_dir(project_dir)

    project_dir_branch_dir = os.path.join(project_dir, branch_name)
    if os.path.exists(os.path.join(project_dir_branch_dir, '.git')):
        pull_git_project(project_dir, branch_name)
    else:
        clone_git_project(git_url, branch_name, project_dir)


def get_gitlab_project_info_by_id(host, token, project_id):
    return json.loads(request.urlopen(
        '%s/api/v4/projects/%s?private_token=%s' % (host, project_id, token)).read())


def get_gitlab_project_info_by_url(web_url, token):
    parse = urlparse(web_url)
    host = parse.netloc
    path = parse.path[1:]
    url = 'http://%s/api/v4/projects/%s?private_token=%s' % (host, quote(path, safe=''), token)
    return json.loads(request.urlopen(url).read())


def download_gitlab_project(host, token, project_id, git_branch, dst_dir):
    download_file(
        "%s/api/v4/projects/%s/repository/archive.zip?sha=%s&private_token=%s" % (
            host,
            project_id, quote(git_branch), token), dst_dir)


def get_dir_files_size(src_path, file_size_dict={}, ignore_dir=''):
    files = os.listdir(src_path)
    for file in files:
        if os.path.splitext(file)[0].startswith('.') or \
                os.path.join(src_path, file) == os.path.join(src_path, ignore_dir):
            # 排除.开头的隐藏文件和目录
            continue
        file_path = os.path.join(src_path, file)
        if os.path.isdir(file_path):
            get_dir_files_size(file_path, file_size_dict)
        else:
            # 去除掉公共目录起始再存入json
            file_size_dict[file_path] = os.path.getsize(file_path)

    return file_size_dict


def create_json_file(dst_path, dist):
    with open(dst_path, 'w+', encoding='utf-8') as f:
        json.dump(dist, f, ensure_ascii=False, indent=4)
